import Phaser from '../lib/phaser.js'
import Laser from './Laser.js'
import Meteor from './Meteor.js'

export default class Game extends Phaser.Scene {
 constructor() {
   super('game') ;
 }

 preload() {
   // chargement de l’image en lui associant un identifiant appelé ‘background’
   this.load.image('background', 'assets/background/nebula.red.png');
   this.load.spritesheet("shooter", "assets/shooter/tiledShooter.png", { frameWidth: 53.5, frameHeight: 60 });
   this.load.spritesheet('laser', 'assets/laser/tiledLaser.png', { frameWidth: 20, frameHeight: 64 });
   this.load.image('meteor', 'assets/meteor/meteor.png');
   this.load.image('meteor2', 'assets/meteor/meteor2.png');
   this.load.spritesheet('meteorExplosion', 'assets/tiledExplosion.png', { frameWidth: 128, frameHeight: 64 });
  }

  create() {
    this.physics.world.setBoundsCollision();

    this.cursors = this.input.keyboard.createCursorKeys();

    this.background = this.add.tileSprite(300,400,600,800,'background');

    this.anims.create({
      key: "fly",
      frameRate: 30,
      frames: this.anims.generateFrameNumbers("shooter", { start: 0, end: 19 }),
      repeat: -1
     });
    
    this.shooter = this.physics.add.sprite(300, 600, 'shooter');
    this.shooter.body.collideWorldBounds = true;
    this.shooter.play("fly");

    this.randomData = new Phaser.Math.RandomDataGenerator(999);
    this.meteorGroup = this.physics.add.group({
     classType: Meteor
    });

    this.meteorTimer = this.time.addEvent( { 
      delay : 1000,
      callback: () => {
       let meteorRandomView =  Math.ceil(Math.random()*2);
       let meteorView = 'meteor';
       if ( meteorRandomView == 1 ) {
        meteorView = 'meteor2';
       }
       this.spawnMeteor(meteorView,0,100,15,this) ;
      },
      loop: false, repeat : 25, callbackScope: this 
     });   
   
    this.physics.add.overlap(this.shooter,this.meteorGroup,this.handleShooterMeteorCollision,undefined,this);

    this.laserGroup = this.physics.add.group({
      classType: Laser
     });
     this.physics.add.overlap(this.laserGroup, this.meteorGroup,this.handleLaserMeteorCollision,undefined,this);
    
  }
   
  update() {        
    this.background.tilePositionY -= 5;

    // player control
    if ( this.cursors.left.isDown) {
        this.shooter.setVelocityX(-200);
    } else if ( this.cursors.right.isDown) {
        this.shooter.setVelocityX(200);
    } else if ( this.cursors.up.isDown) {
        this.shooter.setVelocityY(-200);
    } else if ( this.cursors.down.isDown) {
        this.shooter.setVelocityY(200);
    } else if ( this.cursors.space.isDown ) {
        this.readyToUp = true;          
    } else if ( this.readyToUp && this.cursors.space.isUp ) {
      this.laser = this.laserGroup.get(this.shooter.x, this.shooter.y-this.shooter.height,'laser');
      this.add.existing(this.laser);
      this.laser.velocity = -300;
      this.laser.start();
      this.readyToUp = false;    
    } else {
        this.shooter.setVelocity(0);
    }
  }

  spawnMeteor(meteorView, velocityX, velocityY,angularVelocity,scene) {
    const meteor = this.meteorGroup.get(this.randomData.between(0, 600),50,meteorView);
    scene.add.existing(meteor);
    meteor.start(velocityX, velocityY);
    meteor.anim(angularVelocity);
  }
   
  handleShooterMeteorCollision(shooter,meteor) {
    meteor.disableBody();
    meteor.play('explosion');
    meteor.once(Phaser.Animations.Events.ANIMATION_COMPLETE, () => {
      meteor.destroy();
    });
  }
 
  handleLaserMeteorCollision(shooter,meteor) {
    meteor.disableBody();
    meteor.play('explosion');
    meteor.once(Phaser.Animations.Events.ANIMATION_COMPLETE, () => {
      meteor.destroy();
    });
  }
}