import Phaser from '../lib/phaser.js'
import Laser from './Laser.js'
import Meteor from './Meteor.js'
import HealthBar from './HealthBar.js'

export default class Game extends Phaser.Scene {
 constructor() {
   super('game') ;
 }

 score = 0;
 gameOn = false;

 preload() {
    // images et animations
    this.load.image('background', 'assets/background/nebula.red.png');
    this.load.spritesheet("shooter", "assets/shooter/tiledShooter.png", { frameWidth: 53.5, frameHeight: 60 });
    this.load.spritesheet('laser', 'assets/laser/tiledLaser.png', { frameWidth: 20, frameHeight: 64 });
    this.load.image('meteor', 'assets/meteor/meteor.png');
    this.load.image('meteor2', 'assets/meteor/meteor2.png');
    this.load.spritesheet('meteorExplosion', 'assets/tiledExplosion.png', { frameWidth: 128, frameHeight: 64 });
    this.load.image('play', 'assets/ui/play.png');
    this.load.image('gameOver', 'assets/ui/gameOver.png');


    // audio
    this.load.audio('laser', 'assets/audio/laserSound.mp3');
    this.load.audio('explosion', 'assets/audio/explosion.wav');
  }

  create() {
 
    this.physics.world.setBoundsCollision();

    this.cursors = this.input.keyboard.createCursorKeys();

    this.laserSound = this.sound.add("laser");
    this.explosionSound = this.sound.add("explosion");  

    const style = { color: '#ffffff', fontSize: 15, fontFamily: 'Verdana' };
    this.scoreText = this.add.text(0, 7.5, 'SCORE : 0', style);

    this.play = this.add.image(this.sys.game.canvas.width/2,this.sys.game.canvas.height/3,'play');
    this.play.setVisible(true);        
    this.gameOver = this.add.image(this.sys.game.canvas.width/2,this.sys.game.canvas.height/1.5,'gameOver');
    this.gameOver.setVisible(false);

    this.play.setInteractive();
    this.play.on(Phaser.Input.Events.POINTER_DOWN, () => {
     this.play.setVisible(false);
     this.gameOver.setVisible(false);
     this.shooter.setX(300);
     this.shooter.setY(600);
     this.shooter.setVisible(true);
     this.randomData.sow(999);
     this.score = 0;
     const value = `Score: ${this.score}`;
     this.scoreText.text = value;
     this.healthBar.value = 100;
     this.healthBar.draw();
     this.gameOn = true;
    });
   

    this.playerInfLayer = this.add.layer();
    this.gameLayer = this.add.layer();
    this.playerInfLayer.setDepth(2);
    this.gameLayer.setDepth(1);
    this.healthBar = new HealthBar(this, 0, 392);
    this.playerInfLayer.add([this.scoreText,this.healthBar,this.play,this.gameOver]);

    this.background = this.add.tileSprite(300,400,600,800,'background');

    this.anims.create({
      key: "fly",
      frameRate: 30,
      frames: this.anims.generateFrameNumbers("shooter", { start: 0, end: 19 }),
      repeat: -1
     });
    
    this.shooter = this.physics.add.sprite(300, 600, 'shooter');
    this.shooter.body.collideWorldBounds = true;
    this.shooter.play("fly");

    this.gameLayer.add([this.background,this.shooter]);

    this.randomData = new Phaser.Math.RandomDataGenerator(999);
    this.randomDataMeteor = new Phaser.Math.RandomDataGenerator(999);
    this.meteorGroup = this.physics.add.group({
     classType: Meteor
    });

    this.meteorTimer = this.time.addEvent( { 
      delay : 1000,
      callback: () => {
       let meteorRandomView =  Math.ceil(Math.random()*2);
       let meteorView = 'meteor';
       if ( meteorRandomView == 1 ) {
        meteorView = 'meteor2';
       }
       if ( this.gameOn ) {
        this.spawnMeteor(meteorView,0,100,15,this,this.gameLayer) ;
       }        
      },
      loop: false, repeat : 25, callbackScope: this 
     });   
   
    this.physics.add.overlap(this.shooter,this.meteorGroup,this.handleShooterMeteorCollision,undefined,this);

    this.laserGroup = this.physics.add.group({
      classType: Laser
     });
     this.physics.add.overlap(this.laserGroup, this.meteorGroup,this.handleLaserMeteorCollision,undefined,this);
    
  }
   
  update() {        

    if ( this.gameOn ) {
      this.background.tilePositionY -= 5;

      // player control
      if ( this.cursors.left.isDown) {
          this.shooter.setVelocityX(-200);
      } else if ( this.cursors.right.isDown) {
          this.shooter.setVelocityX(200);
      } else if ( this.cursors.up.isDown) {
          this.shooter.setVelocityY(-200);
      } else if ( this.cursors.down.isDown) {
          this.shooter.setVelocityY(200);
      } else if ( this.cursors.space.isDown ) {
          this.readyToUp = true;          
      } else if ( this.readyToUp && this.cursors.space.isUp ) {
        this.laser = this.laserGroup.get(this.shooter.x, this.shooter.y-this.shooter.height,'laser');
        this.add.existing(this.laser);
        this.laserSound.play() ;
        this.gameLayer.add(this.laser);
        this.laser.velocity = -300;
        this.laser.start();
        this.readyToUp = false;    
      } else {
          this.shooter.setVelocity(0);
      }
    }
  }

  spawnMeteor(meteorView, velocityX, velocityY,angularVelocity,scene,layer) {
    const meteor = this.meteorGroup.get(this.randomData.between(0, 600),50,meteorView);
    scene.add.existing(meteor);
    layer.add(meteor);
    meteor.start(this.randomDataMeteor.between(-200, 200),this.randomDataMeteor.between(20, 300));
    meteor.anim(angularVelocity);
  }
   
  handleShooterMeteorCollision(shooter,meteor) {
    meteor.disableBody();
    this.explosionSound.play();
    meteor.play('explosion');
    meteor.once(Phaser.Animations.Events.ANIMATION_COMPLETE, () => {
      meteor.destroy();
    });
    if ( this.healthBar.decrease(10) ) {
      this.gameOver.setVisible(true);
      this.play.setVisible(true);
      this.shooter.setVisible(false);
      this.meteorGroup.clear(true,true);
      this.gameOn = false;
     }   
  }
 
  handleLaserMeteorCollision(shooter,meteor) {
    meteor.disableBody();
    this.explosionSound.play();
    meteor.play('explosion');
    meteor.once(Phaser.Animations.Events.ANIMATION_COMPLETE, () => {
      meteor.destroy();
    });
    this.score+=10;
    const value = `SCORE: ${this.score}`;
    this.scoreText.text = value;
  }
}